/* an extended version of command with perfex-style usage */
/* by Anton Ertl 2000-12-2, tested with perfctr-1.6 */
/* $Id: command.c,v 1.5 2000/11/21 02:20:01 mikpe Exp $
 * command.c
 *
 * This test program illustrates how a process may use the perfctr
 * inheritance feature of the Linux x86 Performance-Monitoring Counters
 * interface to monitor the execution of an arbitrary command.
 *
 * Usage: ./command path-to-program [options-for-program]
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/wait.h>
#include "libperfctr.h"

int main(int argc, char **argv)
{
    struct perfctr_dev *dev;
    struct vperfctr *parent;
    struct vperfctr_state state;
    struct perfctr_control control;
    struct perfctr_info info;
    unsigned nrctrs; /* > 1 if we have PMCs in addition to a TSC */
    unsigned n;
    int pid;
    int i;

    /*
     * Check that we have a command to run.
     */
    if( argc < 2 ) {
    usage:
	fprintf(stderr, "Usage: perfex [-e event] ... program program-options\n");
	fprintf(stderr, "  Event selectors are specified as hex numbers\n");
	return 1;
    }

    /*
     * Parent sets up the perfctrs.
     */
    if( (dev = perfctr_dev_open()) == NULL ) {
	perror("perfctr_dev_open");
	return 1;
    }
    if( perfctr_info(dev, &info) ) {
	perror("perfctr_info");
	return 1;
    }
    nrctrs = perfctr_cpu_nrctrs(&info);
    if( (parent = vperfctr_attach(dev)) == NULL ) {
	perror("vperfctr_attach");
	return 1;
    }
    memset(&control, 0, sizeof control);
    for (i=1, n=0; strcmp(argv[i],"-e")==0; i+=2) {
      if (argc<i+3)
	goto usage;
      if (n+1<nrctrs) {
	char *endp;
	control.evntsel[n] = strtoul(argv[i+1], &endp, 16);
	if (*endp=='\0')
	  n++;
	else
	  fprintf(stderr,"%s ignored: not a hex number.\n", argv[i+1]);
      } else {
	fprintf(stderr,"%s ignored: not enough counters.\n", argv[i+1]);
      }
    }
    if( vperfctr_control(parent, &control) < 0 ) {
	perror("vperfctr_control");
	return 1;
    }
    perfctr_dev_close(dev);

    /*
     * Fork, and let child exec the command.
     */
    if( (pid = fork()) < 0 ) {
	perror("fork");
	return 1;
    } else if( pid == 0 ) {
	vperfctr_close(parent);
	execvp(argv[i], argv+i);
	perror(argv[i]);
	_exit(1);
    }

    /*
     * Parent waits for child's exit and reports final perfctr values.
     */
    waitpid(pid, NULL, 0); /* XXX: should check for non-successful exits */
    if( vperfctr_read_state(parent, &state) < 0 ) {
	perror("vperfctr_read_state");
	return 1;
    }
    printf(  "tsc           : %19lld\n", state.children.ctr[0]);
    for (i=0; i<n; i++)
      printf("event %08x: %19lld\n",control.evntsel[i],state.children.ctr[i+1]);
    return 0;
}
